using System;
using System.IO;
using System.Linq;
using System.Text;
using NUnit.Framework;

using Smdn.IO;

namespace Smdn.Formats.Mime.Formatting {
  [TestFixture]
  public class MimeMessageBuilderTests {
    private readonly Encoding utf8nobom = new UTF8Encoding(false);

    [Test]
    public void TestDispose()
    {
      var builder = new MimeMessageBuilder();
      var part1 = new MimeMessageBuilder();
      var part2 = new MimeMessageBuilder();

      builder.SubParts.Add(part1);
      builder.SubParts.Add(part2);

      builder.Dispose();

      Assert.Throws<ObjectDisposedException>(() => Assert.IsNotNull(builder.Headers));
      Assert.Throws<ObjectDisposedException>(() => Assert.IsNotNull(builder.SubParts));
      Assert.Throws<ObjectDisposedException>(() => builder.GetHeaders("foo").ToArray());

      const string saveFileName = "test.eml";

      TestUtils.UsingFile(saveFileName, () => {
        Assert.Throws<ObjectDisposedException>(() => builder.Save(saveFileName));
      });

      Assert.Throws<ObjectDisposedException>(() => builder.Save(Stream.Null));

      Assert.Throws<ObjectDisposedException>(() => builder.WriteContent("content", ContentTransferEncodingMethod.SevenBit, Encoding.ASCII));
      Assert.Throws<ObjectDisposedException>(() => builder.WriteContent(Encoding.ASCII.GetBytes("content"), ContentTransferEncodingMethod.SevenBit));
      Assert.Throws<ObjectDisposedException>(() => builder.WriteContent(ContentTransferEncodingMethod.SevenBit, Encoding.ASCII, delegate(StreamWriter writer) {
        writer.WriteLine("content");
      }));
      Assert.Throws<ObjectDisposedException>(() => builder.WriteContent(ContentTransferEncodingMethod.SevenBit, delegate(BinaryWriter writer) {
        writer.Write(0);
      }));
      Assert.Throws<ObjectDisposedException>(() => builder.ToString());


      Assert.Throws<ObjectDisposedException>(() => part1.ToString());
      Assert.Throws<ObjectDisposedException>(() => part2.ToString());


      Assert.DoesNotThrow(() => builder.Dispose(), "re-Dispose");
    }

    [Test]
    public void TestSaveToFile()
    {
      var file = Path.Combine(TestContext.CurrentContext.WorkDirectory, "test.eml");

      using (var builder = new MimeMessageBuilder()) {
        builder.WriteContent("content", ContentTransferEncodingMethod.SevenBit, TestUtils.Encodings.Latin1);

        TestUtils.UsingFile(file, () => {
          builder.Save(file);

          Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                          File.ReadAllText(file));
        });
      }
    }

    [Test]
    public void TestSaveOverwriteExistingFile()
    {
      var file = Path.Combine(TestContext.CurrentContext.WorkDirectory, "test.eml");

      using (var builder = new MimeMessageBuilder()) {
        builder.WriteContent("content", ContentTransferEncodingMethod.SevenBit, TestUtils.Encodings.Latin1);

        TestUtils.UsingFile(file, () => {
          File.WriteAllText(file, new string('*', 1024));

          builder.Save(file);

          Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                          File.ReadAllText(file));
        });
      }
    }

    [Test]
    public void TestSaveToStream()
    {
      using (var builder = new MimeMessageBuilder()) {
        using (var outputStream = new MemoryStream()) {
          builder.WriteContent("content", ContentTransferEncodingMethod.SevenBit, TestUtils.Encodings.Latin1);

          builder.Save(outputStream);

          outputStream.Position = 0L;

          Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                          (new StreamReader(outputStream)).ReadToEnd());
        }
      }
    }

    [Test]
    public void TestWriteContentString()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent("content", ContentTransferEncodingMethod.SevenBit, TestUtils.Encodings.Latin1);

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentByteArray()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(Encoding.ASCII.GetBytes("content"), ContentTransferEncodingMethod.SevenBit);

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentStreamWriterPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.SevenBit, utf8nobom, delegate(StreamWriter writer) {
        writer.Write("content");
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""utf-8""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentStreamWriterNonPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.Base64, utf8nobom, delegate(StreamWriter writer) {
        writer.Write("content");
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Type: text/plain; charset=""utf-8""

Y29udGVudA==".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }
    
    [Test]
    public void TestWriteContentStreamWriterCloseWriterPlainEncoding()
    {
      var builder = new MimeMessageBuilder();
      
      builder.WriteContent(ContentTransferEncodingMethod.SevenBit, utf8nobom, delegate(StreamWriter writer) {
        writer.Write("content");
        writer.Close();
      });
      
      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""utf-8""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentStreamWriterCloseWriterNonPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.Base64, utf8nobom, delegate(StreamWriter writer) {
        writer.Write("content");
        writer.Close();
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Type: text/plain; charset=""utf-8""

Y29udGVudA==".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentBinaryWriterPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.SevenBit, delegate(BinaryWriter writer) {
        var content = Encoding.UTF8.GetBytes("content");

        writer.Write(content, 0, content.Length);
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentBinaryWriterNonPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.Base64, delegate(BinaryWriter writer) {
        var content = Encoding.UTF8.GetBytes("content");

        writer.Write(content, 0, content.Length);
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Type: text/plain; charset=""iso-8859-1""

Y29udGVudA==".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentBinaryWriterCloseWriterPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.SevenBit, delegate(BinaryWriter writer) {
        var content = Encoding.UTF8.GetBytes("content");

        writer.Write(content, 0, content.Length);
        writer.Close();
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Type: text/plain; charset=""iso-8859-1""

content".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }

    [Test]
    public void TestWriteContentBinaryWriterCloseWriterNonPlainEncoding()
    {
      var builder = new MimeMessageBuilder();

      builder.WriteContent(ContentTransferEncodingMethod.Base64, delegate(BinaryWriter writer) {
        var content = Encoding.UTF8.GetBytes("content");

        writer.Write(content, 0, content.Length);
        writer.Close();
      });

      Assert.AreEqual(@"MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Type: text/plain; charset=""iso-8859-1""

Y29udGVudA==".Replace("\r\n", "\n").Replace("\n", "\r\n"),
                      builder.ToString());
    }
  }
}

